@@ -25,17 +25,11 @@
 #include <string.h>
 #include <math.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
-#include <libgnome/gnome-desktop-item.h>
-#include <libgnomeui/gnome-ditem-edit.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include "dlg-properties.h"
 #include "quick-box.h"
 #include "quick-button.h"
 #include "quick-separator.h"
 #include "gtk-utils.h"
-#include "xstuff.h"
 
 #define ICON_SPACING         1       /* spacing between icons */
 #define ICON_SIZE_PANEL      22      /* icons size on panels */
@@ -62,7 +56,6 @@ struct _QuickBoxPrivate {
 
 	PanelAppletOrient  orient;
 	int                size;
-	gboolean           icon_size_follows_panel;
 
 	int                rows;
 	int                columns;
@@ -71,9 +64,10 @@ struct _QuickBoxPrivate {
 	int                min_visible_cols;
 	int                max_visible_cols;
 	int                size_hint[2];
-
+	
 	int                drag_pos_begin;
 	int                drag_pos;
+	QuickDropPos       drop_position;
 	GtkWidget         *popup_menu;
 
 	PopulateMenuFunc   populate_menu_func;
@@ -138,7 +132,8 @@ count_columns (QuickBox *quick_box)
 				col++;
 			col++;
 			row = 0;
-		} else {
+		} 
+		else {
 			row++;
 			if (row == quick_box->priv->rows) {
 				col++;
@@ -237,8 +232,8 @@ quick_box_size_request (GtkWidget        *widget,
 			max_column_width = 0;
 			col++;
 			row = 1;
-
-		} else if ((orientation == GTK_ORIENTATION_HORIZONTAL)
+		} 
+		else if ((orientation == GTK_ORIENTATION_HORIZONTAL)
 			   && QUICK_IS_BUTTON (child_widget)) {
 
 			max_column_width = MAX (max_column_width, 
@@ -256,8 +251,8 @@ quick_box_size_request (GtkWidget        *widget,
 				col++;
 				row = 1;
 			} 
-			
-		} else if ((orientation == GTK_ORIENTATION_VERTICAL)
+		} 
+		else if ((orientation == GTK_ORIENTATION_VERTICAL)
 			   && QUICK_IS_SEPARATOR (child_widget)) {
 			
 			if (max_column_height != 0) {
@@ -283,8 +278,8 @@ quick_box_size_request (GtkWidget        *widget,
 			max_column_height = 0;
 			row++;
 			col = 1;
-
-		} else if ((orientation == GTK_ORIENTATION_VERTICAL)
+		} 
+		else if ((orientation == GTK_ORIENTATION_VERTICAL)
 			   && QUICK_IS_BUTTON (child_widget)) {
 			
 			max_column_height = MAX (max_column_height, 
@@ -311,7 +306,8 @@ quick_box_size_request (GtkWidget        *widget,
 			p->buttons_on_col[col - 1] = row - 1;
 			cur_size += p->col_size[col - 1];
 		}
-	} else {
+	} 
+	else {
 		if (max_column_height != 0) {
 			p->col_size[row - 1] = max_column_height + spacing;
 			p->buttons_on_col[row - 1] = col - 1;
@@ -324,7 +320,8 @@ quick_box_size_request (GtkWidget        *widget,
 			min_size = cur_size;
 		if (col <= max_columns)
 			max_size = cur_size;
-	} else {
+	} 
+	else {
 		if (row <= min_columns)
 			min_size = cur_size;
 		if (row <= max_columns)
@@ -354,7 +351,8 @@ quick_box_size_request (GtkWidget        *widget,
 		requisition->width = min_size;
 		requisition->height = MAX (p->size, arrow_height);
 		p->size_hint[SIZE_HINT_MAX] = max_size - requisition->width;
-	} else {
+	} 
+	else {
 		int last_row = row;
 		if (col == 1)
 			last_row--;
@@ -378,12 +376,13 @@ quick_box_size_request (GtkWidget        *widget,
 	p->size_hint[SIZE_HINT_MIN] = 0;
 
 	/* FIXME */
-	if (! GTK_WIDGET_VISIBLE (arrow)) {
+	/*if (! GTK_WIDGET_VISIBLE (arrow)) {*/
 		if (orientation == GTK_ORIENTATION_HORIZONTAL)
 			p->size_hint[SIZE_HINT_MAX] -= spacing + arrow_width;
 		else
 			p->size_hint[SIZE_HINT_MAX] -= spacing + arrow_height;
-	}
+	/*}*/
+	
 	/**/
 
 	g_list_free (children);
@@ -394,7 +393,7 @@ quick_box_size_request (GtkWidget        *widget,
 	requisition->height += GTK_CONTAINER (box)->border_width * 2;
 
 #ifdef DEBUG
-        g_print ("REQ [%d, %d]", requisition->width, requisition->height);
+        g_print ("REQ [%d, %d]\n", requisition->width, requisition->height);
 #endif
 }
 
@@ -475,18 +474,17 @@ static int
 children_under_the_arrow (QuickBox *quick_box,
 			  int       arrow_size)
 {
-	QuickBoxPrivate *priv = quick_box->priv;
-	int i, c;
 	int n_buttons = 0;
+	int i, c;
 
-	for (c = priv->columns - 1; c >= 0; c--) {
-		arrow_size -= priv->col_size[c];
+	for (c = quick_box->priv->columns - 1; c >= 0; c--) {
+		arrow_size -= quick_box->priv->col_size[c];
 		if (arrow_size <= 0)
 			break;
 	}
 
-	for (i = c; (i >= 0) && (i < priv->columns); i++)
-		n_buttons += priv->buttons_on_col[i];
+	for (i = c; (i >= 0) && (i < quick_box->priv->columns); i++)
+		n_buttons += quick_box->priv->buttons_on_col[i];
 
 	return MAX (0, n_buttons);
 }
@@ -550,8 +548,7 @@ get_child_size (QuickBox *quick_box)
 
 	g_return_val_if_fail (quick_box->priv->rows > 0, 0);
 
-	if (quick_box->priv->icon_size_follows_panel ||
-	    (quick_box->priv->rows == 1))
+	if (quick_box->priv->rows == 1)
 		return quick_box->priv->size;
 
 	child_size = (quick_box->priv->size - (quick_box->priv->rows - 1) * ICON_SPACING) / quick_box->priv->rows;
@@ -576,10 +573,14 @@ quick_box_size_allocate (GtkWidget        *widget,
 	int             max_line_width, max_line_height;
 	int             initial_x, initial_y;
 	GtkRequisition  child_requisition;
-	gboolean        ignoring_arrow, do_not_fit = FALSE;
-	GList          *children, *scan, *hide_from_here = NULL;
+	gboolean        ignoring_arrow;
+	gboolean        do_not_fit = FALSE;
+	GList          *children;
+	GList          *scan;
+	GList          *hide_from_here = NULL;
 	int             under_arrow_children;
-	int             children_to_hide, n_children;
+	int             children_to_hide;
+	int             n_children;
 	gboolean        second_try = FALSE;
 	
 	quick_box = QUICK_BOX (widget);
@@ -604,8 +605,8 @@ quick_box_size_allocate (GtkWidget        *widget,
 
 		/* center vertically */
 		initial_y = (allocation->height - (get_child_size (quick_box) + ICON_SPACING) * quick_box->priv->rows) / 2;
-
-	} else {
+	} 
+	else {
 		/* center horizontally */
 		initial_x = (allocation->width - (get_child_size (quick_box) + ICON_SPACING) * quick_box->priv->rows) / 2;
 
@@ -616,7 +617,7 @@ quick_box_size_allocate (GtkWidget        *widget,
 	max_height = allocation->height;
 
 	if (orientation == GTK_ORIENTATION_HORIZONTAL)
-		max_width  -= arrow_width + spacing;
+		max_width -= arrow_width + spacing;
 	else 
 		max_height -= arrow_height + spacing; 
 
@@ -639,8 +640,7 @@ quick_box_size_allocate (GtkWidget        *widget,
 		GtkWidget     *child_widget = scan->data;
 		GtkAllocation  child_allocation;
 
-		if (ignoring_arrow 
-		    && last_n_child (quick_box, child_widget, under_arrow_children)) {
+		if (ignoring_arrow && last_n_child (quick_box, child_widget, under_arrow_children)) {
 			ignoring_arrow = FALSE;
 			if (orientation == GTK_ORIENTATION_HORIZONTAL) 
 				max_width += arrow_width + spacing;
@@ -664,7 +664,7 @@ quick_box_size_allocate (GtkWidget        *widget,
 
 #ifdef DEBUG
 			g_print ("\n");
-			g_print ("[X] %d <--> %d\n", x + child_allocation.width, max_width);
+			g_print ("[X] (%d + %d) %d <--> %d\n", x, child_allocation.width, x + child_allocation.width, max_width);
 			g_print ("[Y] %d <--> %d\n", current_height, max_height);
 #endif
 			
@@ -673,11 +673,13 @@ quick_box_size_allocate (GtkWidget        *widget,
 					do_not_fit = TRUE;
 					hide_from_here = scan;
 				}
-			} else if (current_height > max_height) { /* new row */
+			} 
+			else if (current_height > max_height) { /* new row */
 				if (second_try) {
 					do_not_fit = TRUE;
 					hide_from_here = scan;
-				} else {
+				} 
+				else { /* new column */
 					x += max_line_width + spacing;
 					y = initial_y;
 					current_height = border_width;
@@ -686,7 +688,8 @@ quick_box_size_allocate (GtkWidget        *widget,
 					continue;
 				}
 			}
-		} else {
+		} 
+		else {
 			
 #ifdef DEBUG
 			g_print ("\n");
@@ -700,11 +703,13 @@ quick_box_size_allocate (GtkWidget        *widget,
 					do_not_fit = TRUE;
 					hide_from_here = scan;
 				}
-			} else if (current_width > max_width) { /* new row */
+			} 
+			else if (current_width > max_width) { /* new row */
 				if (second_try) {
 					do_not_fit = TRUE;
 					hide_from_here = scan;
-				} else {
+				} 
+				else {
 					x = initial_x;
 					y += max_line_height + spacing;
 					current_width = border_width;
@@ -721,7 +726,8 @@ quick_box_size_allocate (GtkWidget        *widget,
 			max_line_width = MAX (max_line_width, child_allocation.width);
 			current_height += spacing;
 			y += child_allocation.width + spacing;
-		} else {
+		} 
+		else {
 			max_line_height = MAX (max_line_height, child_allocation.height);
 			current_width += spacing;
 			x += child_allocation.height + spacing;
@@ -739,7 +745,8 @@ quick_box_size_allocate (GtkWidget        *widget,
 			child_allocation.y = border_width + 1;
 			child_allocation.width  = arrow_width;
 			child_allocation.height = height - 1;
-		} else {
+		} 
+		else {
 			child_allocation.x = border_width + 1;
 			child_allocation.y = allocation->height - arrow_height;
 			child_allocation.width  = width - 1;
@@ -747,21 +754,25 @@ quick_box_size_allocate (GtkWidget        *widget,
 		}
 
 		gtk_widget_size_allocate (arrow, &child_allocation);
+
 		gtk_widget_show (arrow);
 		gtk_widget_set_sensitive (arrow, TRUE);
 
 		quick_box_forall_arrows (quick_box,
 					 (GtkCallback) hide_non_current_arrow_cb,
 					 quick_box);
-	} else 
+	} 
+	else {
 		quick_box_forall_arrows (quick_box,
 					 (GtkCallback) hide_all_arrows_cb,
 					 NULL);
+	}
 	
 	if (do_not_fit) {
 		children_to_hide = MAX (g_list_length (hide_from_here), under_arrow_children);
 		hide_last_n_children (quick_box, children_to_hide);
-	} else 
+	} 
+	else 
 		children_to_hide = 0;
 
 	n_children = g_list_length (children);
@@ -774,7 +785,8 @@ quick_box_size_allocate (GtkWidget        *widget,
 			widget->allocation.width = allocation->width;
 		else
 			widget->allocation.width = x + max_line_width + spacing;
-	} else {
+	} 
+	else {
 		if (do_not_fit)
 			widget->allocation.height = allocation->height;
 		else
@@ -800,7 +812,6 @@ get_drag_destination_rect (QuickBox     *quick_box,
 			   GdkRectangle *rect)
 {
 	GtkWidget *widget = GTK_WIDGET (quick_box);
-	GtkBox    *box = GTK_BOX (quick_box);
 	GList     *children, *scan;
 	int        i, cx, cy, cwidth, cheight;
 
@@ -811,13 +822,19 @@ get_drag_destination_rect (QuickBox     *quick_box,
 	
 	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
 	i = 0;
-	for (scan = children; scan && (i <= pos); scan = scan->next) {
+	for (scan = children; scan; scan = scan->next) {
 		GtkWidget *child_widget = scan->data;
 
-		cx = child_widget->allocation.x;
-		cy = child_widget->allocation.y;
-		cwidth = child_widget->allocation.width;
-		cheight = child_widget->allocation.height;
+		if (! QUICK_IS_BUTTON (child_widget))
+			continue;
+
+		if (i == pos) {
+			cx = child_widget->allocation.x;
+			cy = child_widget->allocation.y;
+			cwidth = child_widget->allocation.width;
+			cheight = child_widget->allocation.height;
+			break;
+		}
 		
 		i++;
 	}
@@ -837,7 +854,7 @@ quick_box_expose (GtkWidget        *widget,
 	QuickBox *quick_box = QUICK_BOX (widget);
 	int       drag_pos;
 	int       drag_pos_begin;
-
+		
 	GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
 
 	drag_pos = quick_box->priv->drag_pos;
@@ -845,32 +862,68 @@ quick_box_expose (GtkWidget        *widget,
 
 	if ((drag_pos != -1) && (drag_pos != drag_pos_begin)) {
 		GdkRectangle r;
-		
+
+#if 0		
+		switch (quick_box->priv->drop_position) {
+		case QUICK_DROP_POSITION_INTO_OR_BEFORE:
+			g_print ("DropPosition: (%d) Into-Or-Before\n", drag_pos);
+			break;
+		case QUICK_DROP_POSITION_INTO_OR_AFTER:
+			g_print ("DropPosition: (%d) Into-Or-After\n", drag_pos);
+			break;
+		case QUICK_DROP_POSITION_BEFORE:
+			g_print ("DropPosition: (%d) Before\n", drag_pos);
+			break;
+		case QUICK_DROP_POSITION_AFTER:
+			g_print ("DropPosition: (%d) After\n", drag_pos);
+			break;
+		}
+#endif
+
 		get_drag_destination_rect (quick_box, 
 					   quick_box->priv->drag_pos, 
 					   &r);
 
-		gdk_draw_line (widget->window,
-			       widget->style->black_gc,
-			       r.x,
-			       r.y + r.height - 1,
-			       r.x + r.width - 1,
-			       r.y + r.height - 1);
-
-		if ((drag_pos_begin != -1) && (drag_pos < drag_pos_begin))
-			gdk_draw_line (widget->window,
-				       widget->style->black_gc,
-				       r.x,
-				       r.y,
-				       r.x,
-				       r.y + r.height - 1);
-		else
-			gdk_draw_line (widget->window,
-				       widget->style->black_gc,
-				       r.x + r.width - 1,
-				       r.y,
-				       r.x + r.width - 1,
-				       r.y + r.height - 1);
+		if (quick_box->priv->drop_position == QUICK_DROP_POSITION_BEFORE) {
+			switch (get_gtk_orientation (quick_box)) {
+			case GTK_ORIENTATION_HORIZONTAL:
+				gdk_draw_line (widget->window,
+					       widget->style->black_gc,
+					       r.x,
+					       r.y,
+					       r.x,
+					       r.y + r.height - 1);
+				break;
+			case GTK_ORIENTATION_VERTICAL:
+				gdk_draw_line (widget->window,
+					       widget->style->black_gc,
+					       r.x,
+					       r.y,
+					       r.x + r.width - 1,
+					       r.y);
+				break;
+			}
+		}
+		else if (quick_box->priv->drop_position == QUICK_DROP_POSITION_AFTER) {
+			switch (get_gtk_orientation (quick_box)) {
+			case GTK_ORIENTATION_HORIZONTAL:
+				gdk_draw_line (widget->window,
+					       widget->style->black_gc,
+					       r.x + r.width - 1,
+					       r.y,
+					       r.x + r.width - 1,
+					       r.y + r.height - 1);
+				break;
+			case GTK_ORIENTATION_VERTICAL:
+				gdk_draw_line (widget->window,
+					       widget->style->black_gc,
+					       r.x,
+					       r.y + r.height - 1,
+					       r.x + r.width - 1,
+					       r.y + r.height - 1);
+				break;
+			}
+		}	
 	}
 
 	if (GTK_WIDGET_HAS_FOCUS (widget) 
@@ -918,9 +971,6 @@ quick_box_finalize (GObject *object)
   
 	quick_box = QUICK_BOX (object);
 
-	if (quick_box->tooltips != NULL) 
-                g_object_unref (quick_box->tooltips);
-
 	g_free (quick_box->priv->col_size);
 	g_free (quick_box->priv->buttons_on_col);
 	g_object_unref (quick_box->priv->icon_theme);
@@ -954,44 +1004,16 @@ quick_box_class_init (QuickBoxClass    *class)
 }
 
 
-static gboolean
-enable_animations (void)
-{
-	GConfClient *client = gconf_client_get_default ();
-	gboolean     global_animation, panel_animation;
-
-	global_animation = gconf_client_get_bool (client, "/desktop/gnome/interface/enable_animations", NULL);
-	panel_animation = gconf_client_get_bool (client, "/apps/panel/global/enable_animations", NULL);
-
-	return global_animation && panel_animation;
-}
-
-
 static gboolean
 item_activated (QuickButton *button)
 {
-	GnomeDesktopItem *ditem;
+	GKeyFile *desktop_entry;
 
-	ditem = g_object_get_data (G_OBJECT (button), "ditem");
-	if (ditem != NULL) {
-		GError  *error = NULL;
-
-		if (enable_animations ())
-			xstuff_zoom_animate (GTK_WIDGET (button), NULL);
-
-		gnome_desktop_item_launch_on_screen (ditem, 
-						     NULL,
-						     0,
-						     gtk_widget_get_screen (GTK_WIDGET (button)),
-						     -1, &error);
-
-		if (error != NULL)
-			_gtk_error_dialog_from_gerror_run (NULL, &error);
-
-		return TRUE;
-	}
-
-	return FALSE;
+	desktop_entry = g_object_get_data (G_OBJECT (button), "desktop_entry");
+	if (desktop_entry != NULL)
+		_g_desktop_entry_launch (desktop_entry, GTK_WIDGET (button), NULL);
+		
+	return TRUE;
 }
 
 
@@ -1088,7 +1110,8 @@ menu_item__drag_begin  (GtkWidget        *widget,
 			gpointer          data)
 {
 	QuickBox *quick_box = data;
-	gtk_tooltips_disable (quick_box->tooltips);
+	
+	gtk_widget_set_has_tooltip (GTK_WIDGET (quick_box), FALSE);
 }
 
 
@@ -1098,7 +1121,8 @@ menu_item__drag_end  (GtkWidget        *widget,
 		      gpointer          data)
 {
 	QuickBox *quick_box = data;
-	gtk_tooltips_enable (quick_box->tooltips);
+	
+	gtk_widget_set_has_tooltip (GTK_WIDGET (quick_box), TRUE);
 }
 
 
@@ -1139,11 +1163,11 @@ static void deactivate_arrow (QuickBox *quick_box);
 static void
 display_popup_menu (QuickBox *quick_box)
 {
-	GtkWidget *popup_menu = NULL;
-	GtkWidget *item;
-	GList     *children = NULL, *scan;
-	gboolean   menu_empty = TRUE;
-
+	GtkWidget  *popup_menu = NULL;
+	GtkWidget  *item;
+	GList      *children = NULL, *scan;
+	gboolean    menu_empty = TRUE;
+	
 	if (quick_box->priv->popup_menu != NULL) {
 		gtk_menu_shell_deactivate (GTK_MENU_SHELL (quick_box->priv->popup_menu));
 		quick_box->priv->popup_menu = NULL;
@@ -1153,20 +1177,20 @@ display_popup_menu (QuickBox *quick_box)
 	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
 
 	for (scan = children; scan; scan = scan->next) {
-		GtkWidget        *child_widget = scan->data;
-		GnomeDesktopItem *ditem;
-		GtkWidget        *icon = NULL;
-		char             *icon_path = NULL;
-		const char       *text;
-		const char       *comment;
-		char             *tip;
+		GtkWidget *child_widget = scan->data;
+		GKeyFile  *desktop_entry;
+		char      *name;
+		char      *icon;
+		GtkWidget *image = NULL;
+		char      *comment;
+		char      *tip;
 
 		if ((child_widget != NULL) && GTK_WIDGET_VISIBLE (child_widget))
 			continue;
 
-		ditem = g_object_get_data (G_OBJECT (child_widget), "ditem");
+		desktop_entry = g_object_get_data (G_OBJECT (child_widget), "desktop_entry");
 
-		if (ditem == NULL) { /* add separator */
+		if (desktop_entry == NULL) { /* add separator */
 			
 			/* do not display the separator if it is the first 
 			 * item in the menu. */
@@ -1193,28 +1217,25 @@ display_popup_menu (QuickBox *quick_box)
 
 		/**/
 
-		text = gnome_desktop_item_get_localestring (ditem, "Name"); 
-		item = gtk_image_menu_item_new_with_label (text);
+		name = g_key_file_get_locale_string (desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_NAME, NULL, NULL); 
+		item = gtk_image_menu_item_new_with_label (name);
 
-		icon_path = gnome_desktop_item_get_icon (ditem, quick_box->priv->icon_theme);
-		if (icon_path != NULL)
-			icon = create_image (quick_box->priv->icon_theme, icon_path, ICON_SIZE_MENU);
+		icon = g_key_file_get_string (desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, NULL);
 		if (icon != NULL)
-			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), icon);
+			image = create_image (quick_box->priv->icon_theme, icon, ICON_SIZE_MENU);
+		if (image != NULL)
+			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 
 		/**/
 
-		comment = gnome_desktop_item_get_localestring (ditem, "Comment"); 
-		if ((comment != NULL) && (*comment != 0))
+		comment = g_key_file_get_locale_string (desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_COMMENT, NULL, NULL);
+		
+		if ((comment != NULL) && (strcmp (comment, "") != 0))
 			tip = g_strdup (comment);
 		else
-			tip = g_strdup (text);
-		gtk_tooltips_set_tip (quick_box->tooltips, 
-				      GTK_WIDGET (item),
-				      tip,
-				      NULL);	
-		g_free (tip);
-
+			tip = g_strdup (name);
+		gtk_widget_set_tooltip_text  (GTK_WIDGET (quick_box), tip);
+		
 		/**/
 
 		if (quick_box->priv->orient == PANEL_APPLET_ORIENT_UP)
@@ -1253,9 +1274,12 @@ display_popup_menu (QuickBox *quick_box)
 				  G_CALLBACK (menu_item__drag_end), 
 				  quick_box);
 
-		/**/
-
 		gtk_widget_show (item);
+		
+		g_free (tip);
+		g_free (comment);
+		g_free (icon);
+		g_free (name);
 	}
 
 	g_list_free (children);
@@ -1265,10 +1289,7 @@ display_popup_menu (QuickBox *quick_box)
 		return;
 	}
 
-	gtk_tooltips_set_tip (quick_box->tooltips, 
-			      get_arrow_widget (quick_box),
-			      _("Hide launchers menu"),
-			      NULL);
+	gtk_widget_set_tooltip_text (GTK_WIDGET (get_arrow_widget (quick_box)), _("Hide launchers menu"));
 
 	g_signal_connect_swapped (G_OBJECT (popup_menu),
 				  "deactivate",
@@ -1308,10 +1329,7 @@ deactivate_arrow (QuickBox *quick_box)
 					 quick_box);
 	
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (arrow), FALSE);
-	gtk_tooltips_set_tip (quick_box->tooltips, 
-			      arrow,
-			      _("Show hidden launchers"),
-			      NULL);
+	gtk_widget_set_tooltip_text (GTK_WIDGET (arrow), _("Show hidden launchers"));
 	
 	g_signal_handlers_unblock_by_func (G_OBJECT (arrow),
 					   G_CALLBACK (arrow_button_toggled),
@@ -1348,10 +1366,7 @@ create_arrow_button (QuickBox     *quick_box,
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	gtk_widget_set_parent (button, GTK_WIDGET (quick_box));
 
-	gtk_tooltips_set_tip (quick_box->tooltips, 
-			      button,
-			      _("Show hidden launchers"),
-			      NULL);
+	gtk_widget_set_tooltip_text (GTK_WIDGET (button), _("Show hidden launchers"));
 	g_signal_connect (G_OBJECT (button), 
 			  "button_press_event",
 			  G_CALLBACK (arrow_button_do_not_eat_button_press), 
@@ -1372,37 +1387,31 @@ quick_box_init (QuickBox *quick_box)
 	GtkBox          *box = GTK_BOX (quick_box);
 	QuickBoxPrivate *priv = quick_box->priv;
 
-	box->spacing     = ICON_SPACING;
+	box->spacing = ICON_SPACING;
 	box->homogeneous = FALSE;
 	gtk_container_set_resize_mode (GTK_CONTAINER (quick_box),
 				       GTK_RESIZE_QUEUE);
 
 	priv = quick_box->priv = g_new0 (QuickBoxPrivate, 1);
-	priv->size                    = ICON_SIZE_PANEL;
-	priv->icon_size_follows_panel = FALSE;
-	priv->orient                  = PANEL_APPLET_ORIENT_DOWN;
-	priv->drag_pos                = -1;
-	priv->drag_pos_begin          = -1;
-	priv->rows                    = 1;
-	priv->min_visible_cols        = DEFAULT_MIN_VISIBLE_COLS;
-	priv->max_visible_cols        = DEFAULT_MAX_VISIBLE_COLS;
-	priv->col_size                = NULL;
-	priv->buttons_on_col          = NULL;
+	priv->size = ICON_SIZE_PANEL;
+	priv->orient = PANEL_APPLET_ORIENT_DOWN;
+	priv->drag_pos = -1;
+	priv->drag_pos_begin = -1;
+	priv->rows = 1;
+	priv->min_visible_cols = DEFAULT_MIN_VISIBLE_COLS;
+	priv->max_visible_cols = DEFAULT_MAX_VISIBLE_COLS;
+	priv->col_size = NULL;
+	priv->buttons_on_col = NULL;
 
 	priv->populate_menu_func = NULL;
 	priv->populate_menu_data = NULL;
 
-	quick_box->tooltips = gtk_tooltips_new ();
-	g_object_ref (quick_box->tooltips);
-	gtk_object_sink (GTK_OBJECT (quick_box->tooltips));
-
-	priv->arrow_up    = create_arrow_button (quick_box, GTK_ARROW_UP);
-	priv->arrow_down  = create_arrow_button (quick_box, GTK_ARROW_DOWN);
-	priv->arrow_left  = create_arrow_button (quick_box, GTK_ARROW_LEFT);
+	priv->arrow_up = create_arrow_button (quick_box, GTK_ARROW_UP);
+	priv->arrow_down = create_arrow_button (quick_box, GTK_ARROW_DOWN);
+	priv->arrow_left = create_arrow_button (quick_box, GTK_ARROW_LEFT);
 	priv->arrow_right = create_arrow_button (quick_box, GTK_ARROW_RIGHT);
 }
 
-
 
 GType
 quick_box_get_type ()
@@ -1458,7 +1467,9 @@ quick_box_get_pointer_position (QuickBox *quick_box,
 		
 		if ((x >= rect.x) && (x <= rect.x + rect.width)
 		    && (y >= rect.y) && (y <= rect.y + rect.height))
+		{
 			return pos;
+		}
 		
 		if ((x >= rect.x) && (x <= rect.x + rect.width))
 			same_col_pos = pos;
@@ -1477,21 +1488,118 @@ quick_box_get_pointer_position (QuickBox *quick_box,
 }
 
 
+static QuickDropPos
+get_drop_position_for_rect (QuickBox     *quick_box, 
+			    GdkRectangle *rect, 
+			    int           x, 
+			    int           y)
+{
+	gboolean before;
+	gboolean inside;
+	float    dx;
+	float    dy;
+	float    border_x;
+	float    border_y;
+	
+	dx = x - rect->x;
+	dy = y - rect->y;
+	border_x = ((float) rect->width) / 4.0;
+	border_y = ((float) rect->height) / 4.0;
+	
+	switch (get_gtk_orientation (quick_box)) {
+	case GTK_ORIENTATION_HORIZONTAL:
+		before = dx < (((float) rect->width) / 2.0);
+		inside = (dx > border_x) && (dx < rect->width - border_x);
+		break;
+	case GTK_ORIENTATION_VERTICAL:
+		before = dy < (((float) rect->height) / 2.0);
+		inside = (dy > border_y) && (dy < rect->height - border_y);
+		break;
+	}
+	
+	if (before) {
+		if (inside) 
+			return QUICK_DROP_POSITION_INTO_OR_BEFORE;
+		else
+			return QUICK_DROP_POSITION_BEFORE;
+	}
+	else {
+		if (inside) 
+			return QUICK_DROP_POSITION_INTO_OR_AFTER;
+		else
+			return QUICK_DROP_POSITION_AFTER;
+	}
+	
+	return QUICK_DROP_POSITION_BEFORE;
+}
+
+
+int
+quick_box_get_drop_position (QuickBox     *quick_box, 
+			     int           x, 
+			     int           y,
+			     QuickDropPos *drop_position)
+{
+	GtkBox *box = GTK_BOX (quick_box);
+	GList  *children, *scan;
+	int     pos;
+
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	if (children == NULL) {
+		if (drop_position != NULL)
+			*drop_position = QUICK_DROP_POSITION_BEFORE;
+		return -1;
+	}
+
+	pos = 0;
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget    *child_widget = scan->data;
+		GdkRectangle  rect = child_widget->allocation;
+		
+		if (! QUICK_IS_BUTTON (child_widget))
+			continue;
+		
+		rect.x      -= box->spacing;
+		rect.y      -= box->spacing;
+		rect.width  += box->spacing * 2;
+		rect.height += box->spacing * 2;
+		
+		if ((x >= rect.x) && (x <= rect.x + rect.width)
+		    && (y >= rect.y) && (y <= rect.y + rect.height))
+		{
+			if (drop_position != NULL) 
+				*drop_position = get_drop_position_for_rect (quick_box, &rect, x, y);
+			g_list_free (children);
+			return pos;
+		}
+
+		pos++;
+	}
+	g_list_free (children);
+	
+	if (drop_position != NULL) 
+		*drop_position = QUICK_DROP_POSITION_AFTER;
+	
+	return pos - 1;
+}
+
+
 static void
-draw_drag_destination (QuickBox *quick_box,
-		       int       pos)
+draw_drag_destination (QuickBox     *quick_box,
+		       int           pos,
+		       QuickDropPos  drop_pos)
 {
 	GtkWidget    *widget = GTK_WIDGET (quick_box);
 	GdkRectangle  r;
 
-	if (pos == quick_box->priv->drag_pos) 
+	if ((pos == quick_box->priv->drag_pos) && (drop_pos == quick_box->priv->drop_position)) 
 		return;
 
-	if (quick_box->priv->drag_pos != -1) {
+	if (pos != quick_box->priv->drag_pos) {
+		quick_button_set_dnd_highlight (QUICK_BUTTON (quick_box_get_child_at_pos (quick_box, quick_box->priv->drag_pos)), FALSE);
 		get_drag_destination_rect (quick_box, 
 					   quick_box->priv->drag_pos,
 					   &r);
-
 		gtk_widget_queue_draw_area (widget, 
 					    r.x,
 					    r.y,
@@ -1499,8 +1607,18 @@ draw_drag_destination (QuickBox *quick_box,
 					    r.height);
 	}
 
-	get_drag_destination_rect (quick_box, pos, &r);
 	quick_box->priv->drag_pos = pos;
+	quick_box->priv->drop_position = drop_pos;
+
+	if ((quick_box->priv->drop_position == QUICK_DROP_POSITION_INTO_OR_BEFORE)
+	    || (quick_box->priv->drop_position == QUICK_DROP_POSITION_INTO_OR_AFTER))
+	{
+		quick_button_set_dnd_highlight (QUICK_BUTTON (quick_box_get_child_at_pos (quick_box, quick_box->priv->drag_pos)), TRUE);
+	}
+	else
+		quick_button_set_dnd_highlight (QUICK_BUTTON (quick_box_get_child_at_pos (quick_box, quick_box->priv->drag_pos)), FALSE);
+		
+	get_drag_destination_rect (quick_box, pos, &r);
 	gtk_widget_queue_draw_area (widget, 
 				    r.x,
 				    r.y,
@@ -1516,13 +1634,14 @@ drag_motion (GtkWidget          *widget,
 	     gint                y,
 	     guint               time)
 {
-	QuickBox *quick_box = QUICK_BOX (widget);
-	int       pos;
-
-	pos = quick_box_get_pointer_position (quick_box, x, y);
+	QuickBox     *quick_box = QUICK_BOX (widget);
+	int           pos;
+	QuickDropPos  drop_pos;
+	
+	pos = quick_box_get_drop_position (quick_box, x, y, &drop_pos);
 	if ((pos == -1) && (_gtk_container_get_n_children (GTK_CONTAINER (widget)) == 0))
 		pos = 0;
-	draw_drag_destination (quick_box, pos);
+	draw_drag_destination (quick_box, pos, drop_pos);
 
 	return TRUE;
 }
@@ -1541,6 +1660,7 @@ drag_leave (GtkWidget          *widget,
 
 	quick_box->priv->drag_pos_begin = -1;
 
+	quick_button_set_dnd_highlight (QUICK_BUTTON (quick_box_get_child_at_pos (quick_box, quick_box->priv->drag_pos)), FALSE);
 	get_drag_destination_rect (quick_box, quick_box->priv->drag_pos, &r);
 	quick_box->priv->drag_pos = -1;
 	gtk_widget_queue_draw_area (widget, 
@@ -1555,61 +1675,69 @@ static void
 button_drag_started_cb (GtkWidget *button,
 			QuickBox  *quick_box)
 {
-	quick_box->priv->drag_pos_begin = quick_box_get_pointer_position (quick_box, button->allocation.x + 1, button->allocation.y + 1);
+	quick_box->priv->drag_pos_begin = quick_box_get_drop_position (quick_box, button->allocation.x + 1, button->allocation.y + 1, NULL);
 }
 
 
-static void
-icon_theme_changed (GtkIconTheme  *icon_theme,
-		    QuickBox       *quick_box)
+void
+quick_box_icon_theme_changed (QuickBox *quick_box)
 {
+	int    child_size;
 	GList *children, *scan;
-	int    child_size = get_child_size (quick_box);
+	
+	child_size = get_child_size (quick_box);
 
 	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
 	for (scan = children; scan; scan = scan->next) {
-		GtkWidget        *child_widget = scan->data;
-		GnomeDesktopItem *ditem;
-		char             *icon_path;
+		GtkWidget *child_widget = scan->data;
+		GKeyFile  *desktop_entry;
+		char      *icon;
 
 		if (! QUICK_IS_BUTTON (child_widget))
 			continue;
 
-		ditem = g_object_get_data (G_OBJECT (child_widget), "ditem");
-		if (ditem == NULL)
+		desktop_entry = g_object_get_data (G_OBJECT (child_widget), "desktop_entry");
+		if (desktop_entry == NULL)
 			continue;
 
-		icon_path = gnome_desktop_item_get_icon (ditem, quick_box->priv->icon_theme);
-		quick_button_set_pixmap (QUICK_BUTTON (child_widget), icon_path, child_size);
-		g_free (icon_path);
+		icon = g_key_file_get_string (desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, NULL);
+		quick_button_set_pixmap (QUICK_BUTTON (child_widget), icon, child_size);
+		g_free (icon);
 	}
 	g_list_free (children);
 }
 
 
 GtkWidget *
-quick_box_add_button (QuickBox     *quick_box,
-		      const char   *uri,
-		      int           pos)
+quick_box_add_button (QuickBox   *quick_box,
+		      const char *uri,
+		      int         pos)
 {
-	GnomeDesktopItem *ditem;
-	GtkWidget        *button;
-	const char       *text;
-	const char       *comment;
-	char             *tip;
-	char             *icon_path;
-	int               child_size;
+	char       *filename;
+	GKeyFile   *desktop_entry;
+	char       *name;
+	GtkWidget  *button;
+	char       *comment;
+	char       *tip;
+	char       *icon_path;
+	int         child_size;
 
-	ditem = gnome_desktop_item_new_from_uri (uri, 0, NULL);
-	if (ditem == NULL) 
+	filename = g_filename_from_uri (uri, NULL, NULL);
+	
+	desktop_entry = g_key_file_new ();
+	if (! g_key_file_load_from_file (desktop_entry, filename, G_KEY_FILE_KEEP_TRANSLATIONS, NULL)) {
+		g_free (filename);
 		return NULL;
+	}
+	_g_key_file_cleanup (desktop_entry);
 
-	text       = gnome_desktop_item_get_localestring (ditem, "Name"); 
-	icon_path  = gnome_desktop_item_get_icon (ditem, quick_box->priv->icon_theme);
+	g_free (filename);
+	
+	icon_path = g_key_file_get_string (desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, NULL);
 	child_size = get_child_size (quick_box);
-	button     = quick_button_new (icon_path, child_size, text, quick_box->priv->icon_theme);
-	g_free (icon_path);
-
+	name = g_key_file_get_locale_string (desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_NAME, NULL, NULL);
+	button = quick_button_new (icon_path, child_size, name, quick_box->priv->icon_theme);
+	
 	if (button == NULL) 
 		return NULL;
 	
@@ -1627,9 +1755,9 @@ quick_box_add_button (QuickBox     *quick_box,
 			  quick_box);
 
 	g_object_set_data_full (G_OBJECT (button), 
-				"ditem", 
-				ditem,
-				(GDestroyNotify) gnome_desktop_item_unref);
+				"desktop_entry", 
+				desktop_entry,
+				(GDestroyNotify) g_key_file_free);
 	g_object_set_data_full (G_OBJECT (button),
 				"uri", 
 				g_strdup (uri),
@@ -1637,16 +1765,17 @@ quick_box_add_button (QuickBox     *quick_box,
 
 	/* tip */
 
-	comment = gnome_desktop_item_get_localestring (ditem, "Comment"); 
-	tip = g_strconcat (text,
+	comment = g_key_file_get_locale_string (desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_COMMENT, NULL, NULL);
+	tip = g_strconcat (name,
 			   ((comment != NULL) && (*comment != 0)) ? "\n" : NULL,
 			   ((comment != NULL) && (*comment != 0)) ? comment: NULL,
 			   NULL);
-	gtk_tooltips_set_tip (quick_box->tooltips, 
-			      button,
-			      tip,
-			      NULL);
+	gtk_widget_set_tooltip_text (button, tip);
+	
 	g_free (tip);
+	g_free (comment);
+	g_free (icon_path);
+	g_free (name);
 
 	/**/
 
@@ -1680,7 +1809,7 @@ quick_box_add_separator (QuickBox *quick_box,
 			  G_CALLBACK (button_button_press_cb), 
 			  NULL);
 
-	g_object_set_data (G_OBJECT (quick_sep), "ditem", NULL);	
+	g_object_set_data (G_OBJECT (quick_sep), "desktop_entry", NULL);	
 
 	gtk_box_pack_start (GTK_BOX (quick_box), quick_sep, 0, 0, 0);
 	if (pos != -1)
@@ -1715,10 +1844,6 @@ quick_box_new (PanelAppletOrient  orient,
 			  "drag_leave",
 			  G_CALLBACK (drag_leave), 
 			  quick_box);
-	g_signal_connect (G_OBJECT (quick_box->priv->icon_theme),
-			  "changed",
-			  G_CALLBACK (icon_theme_changed),
-			  quick_box);
 
 	return GTK_WIDGET (quick_box);
 }
@@ -1775,7 +1900,7 @@ set_button_size_cb (GtkWidget *widget,
 					  quick_box->priv->size);
 	else if (QUICK_IS_BUTTON (widget))
 		quick_button_set_pixmap (QUICK_BUTTON (widget), 
-					 QUICK_BUTTON (widget)->icon_path, 
+					 QUICK_BUTTON (widget)->icon, 
 					 size);
 }
 
@@ -1789,17 +1914,11 @@ quick_box_set_size (QuickBox *quick_box,
 	g_return_if_fail (QUICK_IS_BOX (quick_box));
 
 	quick_box->priv->size = size;
-	if (quick_box->priv->icon_size_follows_panel)
-		quick_box->priv->rows = 1;
-	else
-		quick_box->priv->rows = MAX (1, size / ICON_SIZE_PANEL);
-
+	
 	child_size = get_child_size (quick_box);
-
 	gtk_container_foreach (GTK_CONTAINER (quick_box),
 			       (GtkCallback) set_button_size_cb,
 			       GINT_TO_POINTER (child_size));
-
 	gtk_widget_queue_resize (GTK_WIDGET (quick_box));
 }
 
@@ -1851,21 +1970,23 @@ quick_box_get_max_visible_cols (QuickBox *quick_box)
 
 
 void
-quick_box_set_icon_size_follows_panel (QuickBox *quick_box,
-				       gboolean  value)
+quick_box_set_rows (QuickBox *quick_box,
+		    int       value)
+
 {
 	g_return_if_fail (QUICK_IS_BOX (quick_box));
 
-	quick_box->priv->icon_size_follows_panel = value;
+	quick_box->priv->rows = value;
 	quick_box_set_size (quick_box, quick_box->priv->size);
 }
 
 
 int
-quick_box_get_icon_size_follows_panel (QuickBox *quick_box)
+quick_box_get_rows (QuickBox *quick_box)
 {
-	g_return_val_if_fail (QUICK_IS_BOX (quick_box), FALSE);
-	return quick_box->priv->icon_size_follows_panel;
+	g_return_val_if_fail (QUICK_IS_BOX (quick_box), 0);
+
+	return quick_box->priv->rows;
 }
 
 
@@ -1904,43 +2025,123 @@ quick_box_populate_menu_func (QuickBox          *quick_box,
 
 
 void
-quick_box_update_child (QuickBox          *quick_box,
-			QuickButton       *quick_button,
-			GnomeDesktopItem  *ditem)
+quick_box_update_child (QuickBox    *quick_box,
+			QuickButton *quick_button,
+			GKeyFile    *desktop_entry)
 {
-	const char *text;
-	const char *comment;
-	char       *icon_path;
-	char       *tip;
+	GKeyFile *new_desktop_entry;
+	char     *icon;
+	char     *name;
+	char     *comment;
+	char     *tip;
 
 	g_return_if_fail (QUICK_IS_BOX (quick_box));
 	g_return_if_fail (QUICK_IS_BUTTON (quick_button));
 
-	gnome_desktop_item_ref (ditem);
+	new_desktop_entry = _g_key_file_dup (desktop_entry);
+
 	g_object_set_data_full (G_OBJECT (quick_button), 
-				"ditem", 
-				ditem,
-				(GDestroyNotify) gnome_desktop_item_unref);
+				"desktop_entry", 
+				new_desktop_entry,
+				(GDestroyNotify) g_key_file_free);
 
-	icon_path = gnome_desktop_item_get_icon (ditem, quick_box->priv->icon_theme);
-	quick_button_set_pixmap (quick_button, 
-				 icon_path, 
-				 get_child_size (quick_box));
-	g_free (icon_path);
+	icon = g_key_file_get_string (new_desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, NULL);
+	quick_button_set_pixmap (quick_button, icon, get_child_size (quick_box));
+	
+	name = g_key_file_get_locale_string (new_desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_NAME, NULL, NULL);
+	quick_button_set_text (quick_button, name);
 
-	text = gnome_desktop_item_get_localestring (ditem, "Name"); 
-	quick_button_set_text (quick_button, text);
-
-	comment = gnome_desktop_item_get_localestring (ditem, "Comment"); 
-	tip = g_strconcat (text, 
+	comment = g_key_file_get_locale_string (new_desktop_entry, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_COMMENT, NULL, NULL);
+	tip = g_strconcat (name, 
 			   ((comment != NULL) && (*comment != 0)) ? "\n" : NULL, 
 			   comment, 
 			   NULL);
-	gtk_tooltips_set_tip (quick_box->tooltips, 
-			      GTK_WIDGET (quick_button),
-			      tip,
-			      NULL);
+	gtk_widget_set_tooltip_text (GTK_WIDGET (quick_button), tip);
+
 	g_free (tip);
+	g_free (comment);
+	g_free (name);
+	g_free (icon);
 }
 
 
+int
+quick_box_get_child_position (QuickBox  *quick_box,
+			      GtkWidget *child_to_find)
+{
+	GList    *children, *scan;
+	gboolean  found = FALSE;
+	int       pos;
+
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (pos = 0, scan = children; scan; scan = scan->next) {
+		GtkWidget *child_widget = scan->data;
+		
+		if (! QUICK_IS_BUTTON (child_widget))
+			continue;
+		
+		if (child_widget == child_to_find) {
+			found = TRUE;
+			break;
+		}
+		
+		pos++;
+	}
+	g_list_free (children);
+
+	if (! found)
+		return -1;
+	else
+		return pos;
+}
+
+
+GtkWidget *
+quick_box_get_child_at_pos (QuickBox *quick_box,
+			    int       pos)
+{
+	GtkWidget *child_widget = NULL;
+	GList     *children, *scan;
+	int        i;
+	    
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (i = 0, scan = children; scan; scan = scan->next) {
+		child_widget = scan->data;		
+		if (! QUICK_IS_BUTTON (child_widget))
+			continue;
+			
+		if (pos == i)
+			break;
+			
+		i++;
+	}
+	g_list_free (children);
+	
+	return child_widget;
+}
+
+
+GtkWidget *
+quick_box_get_child_from_uri (QuickBox *quick_box,
+			      char     *uri)
+{
+	GList *children, *scan;
+
+	if (uri == NULL)
+		return NULL;
+
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget  *child_widget = scan->data;
+		const char *uri2;
+
+		uri2 = g_object_get_data (G_OBJECT (child_widget), "uri");
+		if (uri2 == NULL)
+			continue;
+		if (strcmp (uri, uri2) == 0)
+			return child_widget;
+	}
+	g_list_free (children);
+
+	return NULL;
+}
